#include "scenegraph.h"
#include "myglwidget.h"

SceneGraph::SceneGraph()
{
    nodes = QList<RigidBodyTransformation*>();
    views = QList<MyGLWidget*>();
    idCounter = 0;
    sphereIndex = 1;
    boxIndex = 1;
    cylIndex = 1;
    coneIndex = 1;
    torusIndex = 1;

}

void SceneGraph::registerView(MyGLWidget* view){
    views.append(view);
}

RigidBodyTransformation* SceneGraph::addSphere(int tesselation){
    nextSphereName = "Sphere " + sphereIndex;
    //tesselation < 3 makes no sense.
    Sphere* s = new Sphere(idCounter, nextSphereName, tesselation+2);
    idCounter++;
    sphereIndex++;

    qDebug() << "New Sphere added in scenegraph." << nextSphereName;

    RigidBodyTransformation* r = new RigidBodyTransformation(s);
    nodes.append(r);
    notifyViews();
    return r;
}

RigidBodyTransformation* SceneGraph::addBox(int tesselation){

    nextBoxName = "Box " + boxIndex;
    Box* b = new Box(idCounter,nextBoxName,tesselation);
    idCounter++;
    boxIndex++;

    qDebug("New box added in scenegraph");

    RigidBodyTransformation* r = new RigidBodyTransformation(b);
    nodes.append(r);

    qDebug("Notify Views...");
    notifyViews();

    return r;
}

RigidBodyTransformation* SceneGraph::addCylinder(int tesselation){
    qDebug("TODO add cylinder to scenegraph");
    return addBox(1);
}

RigidBodyTransformation* SceneGraph::addCone(int tesselation){
    qDebug("TODO add cone to scenegraph");
    return addBox(tesselation);
}

RigidBodyTransformation* SceneGraph::addTorus(int tesselation){
    qDebug("TODO add torus to scenegraph");
    return addBox(1);
}

void SceneGraph::drawAll(){

    QList<RigidBodyTransformation*>::iterator i;
    for (i = nodes.begin(); i != nodes.end(); ++i){
        (*i)->drawChild();
    }
}

void SceneGraph::addRotation(RigidBodyTransformation *r, QQuaternion diff){
    r->addRotation(diff);
    notifyViews();
}

void SceneGraph::addTranslation(RigidBodyTransformation *r, QVector3D diff){
    r->addTranslation(diff);
    notifyViews();
}

void SceneGraph::notifyViews(){
    QList<MyGLWidget*>::iterator i;
    for (i = views.begin(); i != views.end(); i++){
        (*i)->modelChanged();
    }
}
